from pathlib import Path

from menagerie.utils.cache_handler import CacheHandler


class SiteGen:

    def __init__(self, settings, managers):
        self.settings = settings
        self.shared_info = {}
        self.managers = [manager(self) for manager in managers]
        self.cache = CacheHandler(Path(settings['paths']['cache']), settings)

    def find(self):
        for manager in self.managers:
            manager.find()

    def initialize(self):
        for manager in self.managers:
            manager.initialize()

    def generate(self):
        for manager in self.managers:
            manager.generate()



from pathlib import Path

from menagerie.Items.Managers.AbstractManager import AbstractManager
from menagerie.Items.Static.StaticCSS import StaticCSS
from menagerie.Items.Static.StaticImage import StaticImage
from menagerie.Items.Static.StaticItem import StaticItem
from menagerie.Items.Static.StaticJS import StaticJS


class StaticManager(AbstractManager):
    root_dir = 'static'
    changed_files: dict[str, str] = {}
    item_types = (StaticJS, StaticCSS, StaticImage, StaticItem)

    def __init__(self, site_info) -> None:
        super(StaticManager, self).__init__(site_info)
        self.root_dir = Path(self.gen.settings['paths', 'static'])
        self.gen.shared_info['static_map'] = {}
        self.gen.shared_info['image_sizes'] = {}
        self.gen.shared_info['static_filter'] = self.get_static

    def initialize(self):
        for item in self.items:
            item.initialize()
            if hasattr(item,
                       'out_extension') and item.out_extension is not None and item.out_extension not in item.extensions:
                self.gen.shared_info['static_map'][str(item.in_path.as_posix())] = ''.join(item.out_path.suffixes)

    def generate(self):
        self.gen.settings['out_dir'].mkdir(parents=True, exist_ok=True)
        for item in self.items:
            item.generate()

    def get_static(self, rel_path: str) -> str:
        path = rel_path
        if rel_path in self.gen.shared_info['static_map'].keys():
            path = Path(rel_path).with_suffix(self.gen.shared_info['static_map'].get(rel_path)).as_posix()
        return self.gen.settings['url_prefix'] + str(path)



import os
import sys
from pathlib import Path

from menagerie.Items.Managers.MetaManager import MetaManager
from menagerie.Items.Managers.PageManager import PageManager
from menagerie.Items.Managers.StaticManager import StaticManager
from menagerie.Settings import setup_settings
from menagerie.SiteGen import SiteGen
from menagerie.utils.logger import Logger

__all__ = (
    'generate',
)


def generate(config_path: Path):
    Logger.log_info("Loading Config")
    config = setup_settings(config_path)
    managers = (PageManager, StaticManager, MetaManager)
    parent_folder = config_path.resolve().parent
    os.chdir(str(parent_folder))
    Logger.log_info("Setting Up")
    gen = SiteGen(config, managers)
    Logger.log_info("Beginning Generation")
    Logger.log_info("Finding Content")
    gen.find()
    Logger.log_info("Initializing Meta Data")
    gen.initialize()
    Logger.log_info("Generating Content")
    gen.generate()
    if gen.settings['cache_enabled']:
        gen.cache.dump()
    Logger.log_info("Done!")


if __name__ == "__main__":
    generate(*sys.argv)



from PIL import Image

from menagerie.Items.Static.StaticItem import StaticItem


class StaticImage(StaticItem):
    extensions = ('png', 'jpg', 'jpeg', 'webp', 'gif')
    out_extension = None

    def initialize(self) -> None:
        with Image.open(self.get_path_to_open(), mode='r') as img:
            self.manager.gen.shared_info['image_sizes'][
                str(self.out_path.relative_to(self.manager.gen.settings['out_dir']).as_posix())] = img.size



from abc import ABC, abstractmethod
from pathlib import Path
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from menagerie.Items.Managers.AbstractManager import AbstractManager

from menagerie.utils.logger import Logger

__all__ = ('AbstractItem',)


class AbstractItem(ABC):
    should_cache: bool = True
    content: str | bytes
    root_dir: Path
    byte_mode: bool = False
    in_path: Path
    out_path: Path
    extensions: tuple
    out_extension: str
    ignore_names: list[str] = []

    def __new__(cls, *args, **kwargs):
        if cls.extensions is None:
            raise NotImplementedError("extensions are not implemented")
        return super(AbstractItem, cls).__new__(cls)

    def __init__(self, manager: 'AbstractManager', path: Path):
        self.broken: bool = False
        self.manager: 'AbstractManager' = manager
        self.in_path: Path = path.relative_to(self.manager.root_dir)
        self.out_path: Path = Path(self.manager.gen.settings['out_dir'],
                                   Path(self.manager.root_dir, self.in_path).relative_to(Path(self.manager.root_dir)))
        if hasattr(self, 'out_extension') is True and (self.out_extension is not None):
            self.out_path = self.out_path.with_suffix(f'.{self.out_extension}')

    def get_path_to_open(self) -> Path:
        return Path(self.manager.gen.settings['content_dir'], self.manager.root_dir, self.in_path)

    def get_content(self) -> str | bytes:
        kwargs = {
            'mode': 'rb' if self.byte_mode else 'r'
        }
        if self.byte_mode is False:
            kwargs['encoding'] = 'utf-8'
        with self.get_path_to_open().open(**kwargs) as file:
            return file.read()

    def generate(self) -> None:
        content = self.get_content()
        rel_path = str(self.get_path_to_open().relative_to(self.manager.gen.settings['content_dir']).as_posix())
        should_cache = self.manager.gen.settings['cache_enabled'] and (self.byte_mode is False)
        hit, hashed = self.manager.gen.cache.check_for_hit(rel_path, content) if should_cache else (False, None)
        if hit:
            Logger.log_info(".")
            self.save(self.manager.gen.cache.get_cached_output(self.out_path), minify=False)
        else:
            Logger.log_info("Building: " + str(self.in_path.as_posix()) + ' ➔ ' + str(self.out_path.as_posix()))
            output = self.transform(content)
            self.save(output)
            if should_cache:
                self.manager.gen.cache.memo_output(rel_path, self.out_path, hashed)

    @abstractmethod
    def initialize(self) -> None:
        pass

    def transform(self, content: str | bytes) -> str | bytes:
        return content

    def save(self, new_content: str | bytes, **kwargs) -> None:
        self.out_path.parent.mkdir(parents=True, exist_ok=True)
        kwargs = {
            'mode': 'wb+' if self.byte_mode else 'w+'
        }
        if self.byte_mode is False:
            kwargs['encoding'] = 'utf-8'
        with self.out_path.open(**kwargs) as file:
            file.write(new_content)
